import env from './env';
import ObjectStore from './objectStore';
var InitDB = /** @class */ (function () {
    function InitDB(indexeddb, name, version, callbackObject) {
        // 数据库实例
        this.indexeddb = indexeddb;
        // 数据库名称
        this.name = name;
        // 版本号
        this.version = version;
        // 回调函数
        this.callbackObject = callbackObject;
        // db实例
        this.db = undefined;
        // 创建或者打开数据库
        this.request = this.indexeddb.open(name, version);
        // 绑定函数
        this.request.addEventListener('success', this.handleOpenDBSuccess.bind(this), false);
        this.request.addEventListener('error', this.handleOpenDBError.bind(this), false);
        this.request.addEventListener('upgradeneeded', this.handleOpenUpgradeneeded.bind(this), false);
    }
    /* 打开数据库成功 */
    InitDB.prototype.handleOpenDBSuccess = function (event) {
        if (this.callbackObject.success) {
            this.db = event.target.result;
            this.callbackObject.success.call(this, event);
        }
        if (env === 'development') {
            console.log("\u6253\u5F00\u6570\u636E\u5E93\u6210\u529F\uFF01\nname:    " + this.name + "\nversion: " + this.version);
        }
    };
    /* 打开数据库失败 */
    InitDB.prototype.handleOpenDBError = function (event) {
        if (this.callbackObject.error) {
            console.error(event.target.error.message);
            this.callbackObject.error.call(this, event); // event.target.error
        }
        if (env === 'development') {
            console.log("\u6253\u5F00\u6570\u636E\u5E93\u5931\u8D25\uFF01\nname:    " + this.name + "\nversion: " + this.version);
        }
    };
    /* 更新数据库版本 */
    InitDB.prototype.handleOpenUpgradeneeded = function (event) {
        if (this.callbackObject.upgradeneeded) {
            this.db = event.target.result;
            this.callbackObject.upgradeneeded.call(this, event);
        }
        if (env === 'development') {
            console.log("\u6570\u636E\u5E93\u7248\u672C\u66F4\u65B0\uFF01\nname:    " + this.name + "\nversion: " + this.version);
        }
    };
    /* 关闭数据库 */
    InitDB.prototype.close = function () {
        if (this.db) {
            this.db.close();
            this.db = undefined;
            if (env === 'development') {
                console.log("\u6570\u636E\u5E93\u5173\u95ED\u3002\nname:    " + this.name + "\nversion: " + this.version);
            }
        }
    };
    /**
     * 判断是否有ObjectStore
     * @param { string } objectStoreName: ObjectStore名字
     * @return { boolean }
     */
    InitDB.prototype.hasObjectStore = function (objectStoreName) {
        if (this.db) {
            return this.db.objectStoreNames.contains(objectStoreName);
        }
        return false;
    };
    /**
     * 创建ObjectStore
     * @param { string } objectStoreName: ObjectStore名字
     * @param { string } keyPath        : ObjectStore关键字
     * @param { Array<IndexItem> } indexArray      : 添加索引和键值，name -> 索引， index -> 键值
     * @return { this }
     */
    InitDB.prototype.createObjectStore = function (objectStoreName, keyPath, indexArray) {
        if (this.db) {
            if (!this.hasObjectStore(objectStoreName)) {
                var store = this.db.createObjectStore(objectStoreName, { keyPath: keyPath });
                // 创建索引键值
                if (indexArray) {
                    for (var _i = 0, indexArray_1 = indexArray; _i < indexArray_1.length; _i++) {
                        var item = indexArray_1[_i];
                        store.createIndex(item.name, // 索引
                        item.index // 键值
                        );
                    }
                }
                if (env === 'development') {
                    console.log("\u521B\u5EFA\u4E86\u65B0\u7684ObjectStore\uFF1A" + objectStoreName + "\u3002");
                }
            }
            else {
                console.warn("ObjectStore\uFF1A" + objectStoreName + "\u5DF2\u5B58\u5728\u3002");
            }
        }
        return this;
    };
    /**
     * 删除ObjectStore
     * @param { string } objectStoreName: ObjectStore名字
     * @return { this }
     */
    InitDB.prototype.deleteObjectStore = function (objectStoreName) {
        if (this.db) {
            if (this.hasObjectStore(objectStoreName)) {
                this.db.deleteObjectStore(objectStoreName);
                if (env === 'development') {
                    console.log("\u5220\u9664\u4E86\u65B0\u7684ObjectStore\uFF1A" + objectStoreName + "\u3002");
                }
            }
            else {
                console.warn("ObjectStore\uFF1A" + objectStoreName + "\u4E0D\u5B58\u5728\u3002");
            }
        }
        return this;
    };
    /**
     * 获取操作ObjectStore
     * @param { string } objectStoreName: ObjectStore名字
     * @param { boolean } writeAble     : 只读还是读写
     * @return { ObjectStore }
     */
    InitDB.prototype.getObjectStore = function (objectStoreName, writeAble) {
        if (writeAble === void 0) { writeAble = false; }
        if (this.db) {
            return new ObjectStore(this.db, objectStoreName, writeAble);
        }
    };
    return InitDB;
}());
export default InitDB;
