import getRange from './getRange';
import env from './env';
/**
 * 操作objectStore
 * @param { IDBDatabase } db       : db实例
 * @param { string } objectStoreNam: objectStore名字
 * @param { boolean } writeAble    : 是否只读
 */
var ObjectStore = /** @class */ (function () {
    function ObjectStore(db, objectStoreName, writeAble) {
        this.db = db;
        var wa = writeAble === true ? 'readwrite' : 'readonly';
        var transaction = this.db.transaction(objectStoreName, wa);
        this.store = transaction.objectStore(objectStoreName);
        return this;
    }
    /**
     * 添加数据
     * @param { object | Array<Object> } arg: 数组添加多个数据，object添加单个数据
     * @return { this }
     */
    ObjectStore.prototype.add = function (arg) {
        var data = arg instanceof Array ? arg : [arg];
        for (var i = 0, j = data.length - 1; i <= j; i++) {
            this.store.add(data[i]);
            if (i === j && env === 'development') {
                console.log('数据添加成功');
            }
        }
        return this;
    };
    /**
     * 更新数据
     * @param { object | Array<object> } arg: 数组添加多个数据，object添加单个数据
     * @return { this }
     */
    ObjectStore.prototype.put = function (arg) {
        var data = arg instanceof Array ? arg : [arg];
        for (var i = 0, j = data.length - 1; i <= j; i++) {
            this.store.put(data[i]);
            if (i === j && env === 'development') {
                console.log('数据更新成功');
            }
        }
        return this;
    };
    /**
     * 删除数据
     * @param { string | number | Array<string | number> } arg: 数组删除多个数据，string、number删除单个数据
     * @return this
     */
    ObjectStore.prototype.delete = function (arg) {
        var data = arg instanceof Array ? arg : [arg];
        for (var i = 0, j = data.length - 1; i <= j; i++) {
            this.store.delete(data[i]);
            if (i === j && env === 'development') {
                console.log('数据删除成功');
            }
        }
        return this;
    };
    /* 清除数据 */
    ObjectStore.prototype.clear = function () {
        this.store.clear();
        if (env === 'development') {
            console.log('数据清除成功');
        }
        return this;
    };
    /**
     * 获取数据
     * @param { string | number} value: 键值
     * @param { Function } callback   : 获取成功的回调函数
     * @return { this }
     */
    ObjectStore.prototype.get = function (value, callback) {
        var _this = this;
        var idbRequest = this.store.get(value);
        // 成功后的回调函数
        var handleSuccess = function (event) {
            if (callback)
                callback.call(_this, event);
        };
        idbRequest.addEventListener('success', handleSuccess, false);
        return this;
    };
    /**
     * 游标
     * @param { string } indexName               : 索引名
     * @param { string | number | boolean } range: 查询范围：有等于，大于等于，小于，小于等于，区间
     * @param { Function } callback              : 查询成功的回调函数
     * @return { this }
     * result.value
     * result.continue()
     */
    ObjectStore.prototype.cursor = function (indexName) {
        var _this = this;
        // 获取参数
        var callback = typeof arguments[1] === 'function' ? arguments[1] : arguments[2];
        var range = arguments[2] ? getRange(arguments[1]) : undefined;
        var index = this.store.index(indexName);
        var cursor = !range ? index.openCursor() : index.openCursor(range);
        // 成功后的回调函数
        var handleSuccess = function (event) {
            if (callback)
                callback.call(_this, event); // event.target.result.value && event.target.result.continue()
        };
        cursor.addEventListener('success', handleSuccess, false);
        return this;
    };
    return ObjectStore;
}());
export default ObjectStore;
